/*
  str2xbm.c  --  convert string to X bitmap (xbm)
  Copyright (C) 2004 Steffen Solyga <solyga@absinth.net>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
  This program uses the md4 ("RSA Data Security Inc. MD4 Message-Digest
  Algorithm") reference implementation published with RFC 1320.
  For global.h, md4.h and md4c.c:
  Copyright (C) 1990-2, RSA Data Security, Inc. All rights reserved
*/

/*
 *	$Id: str2xbm.c,v 1.2 2004/04/06 21:43:26 solyga Exp $
 */

#include	"header.h"


int
display_help( char* pn ) {
  fprintf( HELP_CHANNEL, "%s v%s (%s): ", pn, VERSION_NUMBER, DATE_OF_LAST_MOD );
  fprintf( HELP_CHANNEL, "Convert string to X bitmap (xbm).\n" );
  fprintf( HELP_CHANNEL, "Flowers & bug reports to %s.\n", MY_EMAIL_ADDRESS );
  fprintf( HELP_CHANNEL, "Usage: %s [options] string\n", pn );
  fprintf( HELP_CHANNEL, "switches:\n" );
  fprintf( HELP_CHANNEL, "  -h\t write this info to %s and exit sucessfully\n", HELP_CHANNEL==stdout?"stdout":"stderr" );
  fprintf( HELP_CHANNEL, "  -V\t print version and compilation info to %s and exit sucessfully\n", VERSION_CHANNEL==stdout?"stdout":"stderr" );
  return( 0 );
}


int
display_version( char* pn ) {
  fprintf( VERSION_CHANNEL, "%s v%s (%s)\n", pn, VERSION_NUMBER, DATE_OF_LAST_MOD );
  return( 0 );
}


char*
str_dsc( char* str1, char* str2) {
/*
returns pointer to first char of str1 not contained in str2
started 1997-02-09
*/
  char *p1, *p2;
  for ( p1=str1; ; p1++ ) {
    if ( *p1 == '\0' ) return( p1 );
    for ( p2=str2; ; p2++ ) {
      if ( *p2 == '\0' ) return( p1 );
      if ( *p2 == *p1 ) break;
    }
  }
}


char*
basename( char* name ) {
/*
 * strip directory from name
 * returns pointer to stripped name
 * hint: basename("/usr/bin/") == ""
 *   basename(1) would return "bin" !!
 */
  char* p= name;
  while( *p != '\0' ) p++;
  while( p > name ) {
    if( *(p-1) == '/' ) break;
    else p--;
  }
  return( p );
}



int
main( int argc, char** argv ) {
/*
 * main() str2xbm
 * started      : Tue Mar 23 23:57:43 CET 2004 @beast
 */
  char* fpn= *argv;
  char* pn= basename( fpn );		/* prg_name is Makefile constant */
  int retval= RETVAL_OK;
  int c;

/* process options */
  *argv= pn;				/* give getop() the cut name */
  while( (c=getopt(argc,argv,"hV")) != EOF ) {
    switch( c ) {
      case 'h': /* display help and exit sucessfully */
        display_help( pn );
        retval= RETVAL_OK; goto DIE_NOW;
      case 'V': /* display version to VERSION_CHANNEL and exit sucessfully */
        display_version( pn );
        retval= RETVAL_OK; goto DIE_NOW;
      case '?': /* refer to -h and exit unsucessfully */
        fprintf( ERROR_CHANNEL, "%s: Try '%s -h' for more information.\n",
                 pn, pn );
        retval= RETVAL_ERROR; goto DIE_NOW;
      default : /* program error */
        fprintf( ERROR_CHANNEL, "%s: Options bug! E-mail me at %s.\n",
                 pn, MY_EMAIL_ADDRESS );
        retval= RETVAL_BUG; goto DIE_NOW;
    }
  }

/* do the work */
  for( ; optind<argc; optind++ ) {
    char* start= argv[optind];
    int len= strlen( start );
    char* p;
    int row;
    /* check string */
    for( p=start; *p!='\0'; p++ ) {
      if( *p < bits_off0  ||  *p >= bits_off1 ) {
        fprintf( ERROR_CHANNEL,
                 "%s: Unknown character '%c' in argument '%s'.\n",
                 pn, *p, argv[optind] );
        retval= RETVAL_INVAL; goto DIE_NOW;
      }
    }
    /* output header */
    fprintf( STD_CHANNEL, "#define count_width %d\n", 8*len );
    fprintf( STD_CHANNEL, "#define count_height 10\n\n" );
    fprintf( STD_CHANNEL, "static char count_bits[] = {\n" );
    /* output bitmap row by row */
    for( row= 0; row<10; row++ ) {
      for( p=start; *p!='\0'; p++ ) {
        fprintf( STD_CHANNEL, " 0x%02x,", bits[*p-bits_off0][row] );
      }
      fprintf( STD_CHANNEL, "\n" );
    }
    /* output trailer */
    fprintf( STD_CHANNEL, "};\n" );
  }

DIE_NOW:
  exit( retval );
}
